import errno
import os

import pkg_resources

from fabric.api import env
from fabric.api import local
from fabric.api import puts
from fabric.api import task


def _makedir(p):
  try:
    os.mkdir(p)
    puts('[files] Making directory: %s' % p)
  except OSError as exc:
      if exc.errno == errno.EEXIST and os.path.isdir(p):
          pass



@task
def init(force=False):
  """Copy the template files to the local directory for easy modification."""

  #_makedir(env.get('farmboy_files'))
  _makedir('./files')

  top_level = pkg_resources.resource_listdir(__name__, '_files')
  for path in top_level:
    if not pkg_resources.resource_isdir(__name__, '_files/%s' % path):
      continue

    _makedir('./files/%s' % path)

    for sub in pkg_resources.resource_listdir(__name__, '_files/%s' % path):
      new_path = './files/%s/%s' % (path, sub)
      if not force and os.path.exists(new_path):
        continue

      old_path = pkg_resources.resource_filename(__name__,
                                                 '_files/%s/%s' % (path, sub))

      local('cp %s %s' % (old_path, new_path))



# vim: tabstop=4 shiftwidth=4 softtabstop=4
import os

import fabtools.files
import fabtools.require

from farmboy import util


from fabric.api import env
from fabric.api import execute
from fabric.api import parallel
from fabric.api import put
from fabric.api import roles
from fabric.api import sudo
from fabric.api import task

from farmboy import gunicorn


@roles('web')
@task
@parallel
def deploy(path=None, use_gunicorn=True):
    """Put a django site onto the web servers."""
    if path is None:
        path = env.farmboy_django_app

    fabtools.require.python.package('django', use_sudo=True)

    project = os.path.basename(path)
    fabtools.require.directory(
            path = util.home('www'),
            owner = env.farmboy_user,
            group = env.farmboy_user,
            use_sudo = True)

    put(local_path = path,
        remote_path = util.home('www'),
        use_sudo = True)

    sudo('chown -R %s:%s /home/%s/www/%s' % (env.farmboy_user,
                                             env.farmboy_user,
                                             env.farmboy_user,
                                             project))

    if use_gunicorn:
        fabtools.require.files.template_file(
            template_source = util.files('gunicorn/django'),
            path     = '/etc/gunicorn.d/django',
            context  = {'project': project},
            owner    = 'root',
            group    = 'root',
            mode     = '644',
            use_sudo = True)

        gunicorn.restart()



# vim: tabstop=4 shiftwidth=4 softtabstop=4
import functools
import os
import sys

import pkg_resources
import yaml


from fabric.api import env
from fabric.utils import puts
from fabric.utils import warn


def files(s):
    """Return file path to a file in shared files path or locally defined.

    Check whether the file exists in the defined path otherwise
    fallback to the one shipped with Farm Boy.
    """
    path = os.path.join(env.get('farmboy_files'), s)
    if os.path.exists(path):
        return path

    # TODO(termie): use pkg_resources or whatnot to get the path to the
    #               files we installed with the package
    path = pkg_resources.resource_filename(__name__, '_files/%s' % s)
    return path

def home(s):
    return os.path.join('/home/%s' % env.get('farmboy_user'), s)


def load_roledefs(path='farmboy.yaml'):
    """Return roledefs from a yaml file.

    Looks first for the 'roledefs' key, otherwise
    assume the roledefs are root elements.
    """
    try:
        doc = yaml.load(open(path))
        return doc.get('roledefs', doc)
    except IOError:
        # log.warn('No file found at %s' % path)
        return {}


def load(key, path='farmboy.yaml'):
    """Return key from a yaml file."""
    try:
        doc = yaml.load(open(path))
        return doc.get(key, None)
    except IOError:
        # log.warn('No file found at %s' % path)
        return None


def update(d, path='farmboy.yaml'):
    """Update various keys in a yaml file."""
    try:
        doc = yaml.load(open(path))
    except IOError:
        doc = {}

    doc.update(d)
    yaml.dump(d,
              stream=open(path, 'w'),
              default_flow_style=False,
              indent=2,
              width=72)


def template_file(source, target, context=None):
    context = context and context or {}
    if not os.path.exists(source):
        source = files(source)

    puts('[local] Templating: %s -> %s' % (source, target))

    template = open(source).read()
    open(target, 'w').write(template % context)


def host(s):
    """Try to get a host out of something like username@ipaddress."""
    return s.split('@')[-1]


# decorator
class requires(object):
    """Throw an error if missing a requirement."""

    def __init__(self, module, package):
        self._module = module
        self._package = package

    def __call__(self, f):
        if self._module not in sys.modules:
            @functools.wraps(f)
            def _requires(*args, **kw):
                raise Exception('This task requires an additional package: '
                                '%s' % (self._package))
            return _requires
        return f



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Original code based on https://github.com/ailling/jenkins-ubuntu-deployment/
# and https://github.com/termie/jenkins-ubuntu-deployment/

import fabtools.require

from fabric.api import env
from fabric.api import execute
from fabric.api import local
from fabric.api import sudo
from fabric.api import roles
from fabric.api import run
from fabric.api import task


@roles('ci')
@task
def jenkins_repo():
    fabtools.require.files.file(
        url = 'http://pkg.jenkins-ci.org/debian/jenkins-ci.org.key',
        path = '/tmp/jenkins-ci.org.key')
    sudo('apt-key add /tmp/jenkins-ci.org.key')
    fabtools.require.files.file(
        contents='deb http://pkg.jenkins-ci.org/debian binary/',
        path='/etc/apt/sources.list.d/jenkins.list',
        use_sudo=True)


@roles('ci')
@task
def depends():
    fabtools.deb.update_index(quiet=False)


@roles('ci')
@task
def jenkins():
    fabtools.require.deb.packages([
        'jenkins',
    ])


@roles('ci')
@task
def nginx():
    fabtools.require.deb.packages([
        'nginx',
    ])
    fabtools.require.file(
        source   = 'files/nginx/jenkins',
        path     = '/etc/nginx/sites-available/jenkins',
        owner    = 'root',
        group    = 'root',
        mode     = '755',
        use_sudo = True)
    sudo('rm -f /etc/nginx/sites-enabled/jenkins')
    sudo('ln -s /etc/nginx/sites-available/jenkins'
         ' /etc/nginx/sites-enabled/jenkins')
    sudo('service nginx restart')


def setup_ssh_keys():
    sudo('ssh-keygen -t rsa', user='jenkins')
    run('echo "your deploy key:"')
    sudo('cat /var/lib/jenkins/.ssh/id_rsa.pub')


@roles('ci')
@task
def deploy():
    """Deploy a Jenkins server to the <ci> host."""
    #setup_machine()
    #setup_apache()
    #if auth_username:
    #    setup_basic_auth_user(auth_username)
    #upload_config(auth_username)
    #enable_apache_site()
    #setup_ssh_keys()
    execute(jenkins_repo)
    execute(depends)
    execute(jenkins)
    execute(nginx)


@roles('ci')
@task
def test_clone_repo(repourl=''):
    if not repourl:
        print 'Specify a repository url'
        return

    path = '/var/lib/jenkins/test_clone'
    sudo('rm -Rf %s' % path)
    sudo('mkdir %s' % path, user='jenkins')
    sudo('chown jenkins:nogroup %s' % path)
    sudo('cd %s; git clone %s' % (path, repourl), user='jenkins')
    sudo('rm -Rf %s' % path)


__all__ = ['deploy']



# vim: tabstop=4 shiftwidth=4 softtabstop=4

import os

from farmboy import aptcacher
from farmboy import aws
from farmboy import core
from farmboy import django
from farmboy import files
from farmboy import gitlab
from farmboy import gunicorn
from farmboy import haproxy
from farmboy import jenkins
from farmboy import openstack
from farmboy import nginx
from farmboy import tomcat
from farmboy import util
from farmboy import vagrant


from fabric.api import env
from fabric.api import execute
from fabric.api import task



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Original code based on https://github.com/AndrewWalker/fabric-gitlab
# and https://github.com/termie/fabric-gitlab

import fabtools.deb
import fabtools.require
import fabtools.files
import fabtools.user

from fabric.api import env
from fabric.api import execute
from fabric.api import local
from fabric.api import roles
from fabric.api import run
from fabric.api import sudo
from fabric.api import task
from fabric.context_managers import cd


@roles('vcs')
@task
def depends():
    """Update dependencies for GitLab on the <vcs> host."""
    fabtools.deb.update_index(quiet=False)
    fabtools.require.deb.packages([
        'build-essential',
        'zlib1g-dev',
        'libyaml-dev',
        'libssl-dev',
        'libgdbm-dev',
        'libreadline-dev',
        'libncurses5-dev',
        'libffi-dev',
        'curl',
        'git-core',
        'openssh-server',
        'redis-server',
        'checkinstall',
        'libxml2-dev',
        'libxslt-dev',
        'libcurl4-openssl-dev',
        'libicu-dev',
        # required for mail notifications
        'postfix',
        # obvs
        'git',
        #'libtool',
        # preferred server
        'nginx'
    ])


@roles('vcs')
@task
def ruby(force=False):
    # The AWS version of Ubuntu 13.04 doesn't ship with ruby, but the vagrant
    # version does...
    fabtools.require.deb.packages([
        'ruby',
    ])
    s = run('ruby --version')
    if force or not s.startswith('ruby 1.9.3p327'):
        sudo('rm -rf /tmp/ruby')
        fabtools.require.directory(
            path    = '/tmp/ruby')

        fabtools.require.file(
            url  = 'http://mirrors.ibiblio.org/ruby/1.9/ruby-1.9.3-p327.tar.gz',
            path = '/tmp/ruby/ruby-1.9.3-p327.tar.gz')

        with cd('/tmp/ruby'):
            run('tar zxvf ruby-1.9.3-p327.tar.gz')

        with cd('/tmp/ruby/ruby-1.9.3-p327'):
            run('./configure')
            run('make')
            sudo('make install')


@roles('vcs')
@task
def bundler():
    sudo('gem install bundler')


@roles('vcs')
@task
def git_user():
    # do this manually, rather than via fabtools
    if not fabtools.user.exists('git'):
        sudo("adduser --disabled-login --gecos 'GitLab' git")


@roles('vcs')
@task
def gitlab_shell():
    if not fabtools.files.is_dir('/home/git/gitlab-shell'):
        with cd('/home/git'):
            sudo('git clone https://github.com/gitlabhq/gitlab-shell.git',
                 user='git')
        with cd('/home/git/gitlab-shell'):
            sudo('git checkout v1.4.0', user='git')

    fabtools.require.files.file(
        source   = 'files/gitlab-shell/config.yml',
        path     = '/home/git/gitlab-shell/config.yml',
        owner    = 'git',
        group    = 'git',
        mode     = '644',
        use_sudo = True)

    with cd('/home/git/gitlab-shell'):
        sudo('./bin/install', user='git')


@roles('vcs')
@task
def database_postgres():
    fabtools.require.deb.packages([
        'postgresql-9.1',
        'libpq-dev'
    ])

    fabtools.require.files.file(
        source  = 'files/postgres/commands',
        path    = '/tmp/commands')

    fabtools.require.files.file(
        source   = 'files/postgres/pg_hba.conf',
        path     = '/etc/postgresql/9.1/main/pg_hba.conf',
        owner    = 'postgres',
        group    = 'postgres',
        mode     = '644',
        use_sudo = True)
    fabtools.require.files.file(
        source   = 'files/postgres/postgresql.conf',
        path     = '/etc/postgresql/9.1/main/postgresql.conf',
        owner    = 'postgres',
        group    = 'postgres',
        mode     = '644',
        use_sudo = True)
    sudo('service postgresql restart')


    # execute psql statements
    sudo('psql -d template1 -f /tmp/commands', user='postgres')


@roles('vcs')
@task
def clone_source():
    if not fabtools.files.is_dir('/home/git/gitlab'):
        with cd('/home/git'):
            sudo('git clone https://github.com/gitlabhq/gitlabhq.git gitlab',
                 user='git')

    with cd('/home/git/gitlab'):
        sudo('git checkout 5-3-stable', user='git')


@roles('vcs')
@task
def configure():
    with cd('/home/git/gitlab'):
        fabtools.require.files.file(
            source   = 'files/gitlab/gitlab.yml',
            path     = '/home/git/gitlab/config/gitlab.yml',
            owner    = 'git',
            group    = 'git',
            mode     = '644',
            use_sudo = True)

        sudo('chown -R git log/')
        sudo('chown -R git tmp/')
        sudo('chmod -R u+rwX  log/')
        sudo('chmod -R u+rwX  tmp/')
        sudo('cp config/puma.rb.example config/puma.rb', user='git')

    fabtools.require.directory(
        path     = '/home/git/gitlab-satellites',
        owner    = 'git',
        group    = 'git',
        mode     = '755',
        use_sudo = True)

    fabtools.require.directory(
        path     = '/home/git/gitlab/tmp/pids',
        owner    = 'git',
        group    = 'git',
        mode     = '755',
        use_sudo = True)

    fabtools.require.directory(
        path     = '/home/git/gitlab/tmp/sockets',
        owner    = 'git',
        group    = 'git',
        mode     = '755',
        use_sudo = True)

    fabtools.require.directory(
        path     = '/home/git/gitlab/public/uploads',
        owner    = 'git',
        group    = 'git',
        mode     = '755',
        use_sudo = True)

    # this makes sense to be in the git_user area, but the install docs
    # have it here, keeping it here to prevent confusion
    fabtools.require.files.file(
        source   = 'files/git/gitconfig.gitlab',
        path     = '/home/git/.gitconfig',
        owner    = 'git',
        group    = 'git',
        mode     = '644',
        use_sudo = True)


@roles('vcs')
@task
def configure_gitlab_db_postgres():
    # this makes sense to be in the database_postgres area, but the install
    # docs have it here, keeping it here to prevent confusion
    fabtools.require.files.file(
        source   = 'files/gitlab/database.yml',
        path     = '/home/git/gitlab/config/database.yml',
        owner    = 'git',
        group    = 'git',
        mode     = '644',
        use_sudo = True)


@roles('vcs')
@task
def install_gems_postgres():
    with cd('/home/git/gitlab'):
        sudo("gem install -V charlock_holmes --version '0.6.9.4'")
        sudo('bundle install --deployment --without development test mysql',
             user='git')


@roles('vcs')
@task
def init_database():
    with cd('/home/git/gitlab'):
        sudo('force=yes bundle exec rake gitlab:setup RAILS_ENV=production',
             user='git')


@roles('vcs')
@task
def init_script():
    with cd('/home/git/gitlab'):
        sudo('cp lib/support/init.d/gitlab /etc/init.d/gitlab')

    sudo('chmod +x /etc/init.d/gitlab')
    sudo('update-rc.d gitlab defaults 21')


@roles('vcs')
@task
def start_gitlab():
    sudo('service gitlab start')


@roles('vcs')
@task
def stop_gitlab():
    sudo('service gitlab stop')


@roles('vcs')
@task
def check_info():
    with cd('/home/git/gitlab'):
        sudo('bundle exec rake gitlab:env:info RAILS_ENV=production',
             user='git')


@roles('vcs')
@task
def check_status():
    with cd('/home/git/gitlab'):
        sudo('bundle exec rake gitlab:check RAILS_ENV=production',
             user='git')


@roles('vcs')
@task
def sidekiq():
    with cd('/home/git/gitlab'):
        sudo('bundle exec rake sidekiq:start RAILS_ENV=production',
             user='git')


@roles('vcs')
@task
def site_configuration():
    fabtools.require.file(
        source   = 'files/nginx/gitlab',
        path     = '/etc/nginx/sites-available/gitlab',
        owner    = 'root',
        group    = 'root',
        mode     = '755',
        use_sudo = True)
    sudo('rm -f /etc/nginx/sites-enabled/gitlab')
    sudo('ln -s /etc/nginx/sites-available/gitlab'
         ' /etc/nginx/sites-enabled/gitlab')
    sudo('service nginx restart')


@roles('vcs')
@task(default=True)
def deploy():
    """Deploy a full GitLab instance to the <vcs> host.

    Because this is a rather large app, be aware that this instance
    should have at least 1GB of RAM associated with it.
    """

    execute(depends)
    execute(ruby)
    execute(bundler)
    execute(git_user)
    execute(gitlab_shell)
    execute(database_postgres)
    execute(clone_source)
    execute(configure)
    execute(configure_gitlab_db_postgres)
    execute(install_gems_postgres)
    execute(init_database)
    execute(init_script)
    execute(start_gitlab)
    #execute(sidekiq)
    execute(site_configuration)


__all__ = ['deploy', 'depends']



import os

import pkg_resources

from fabric import main as fab


def main():
    """Load our default fabfile, then attempt to load any local fabfiles."""
    our_fab = pkg_resources.resource_filename(__name__, 'default_tasks.py')
    #our_fab = os.path.join(os.path.dirname(__file__), 'default_tasks.py')
    docstring, callables, default = fab.load_fabfile(our_fab)
    fab.state.commands.update(callables)


    fabfiles = []
    other_fab = fab.find_fabfile()
    if other_fab:
      fabfiles.append(other_fab)

    fabfiles.append(our_fab)


    fab.main(fabfiles)



# vim: tabstop=4 shiftwidth=4 softtabstop=4
import fabtools.files
import fabtools.require


from fabric.api import execute
from fabric.api import parallel
from fabric.api import roles
from fabric.api import sudo
from fabric.api import task


@roles('web')
@task
def deploy():
    """Add gunicorn to the <web> hosts."""
    fabtools.require.deb.packages([
        'gunicorn'
    ])
    restart()


#@roles('web')
#@task
def restart():
    """Gunicorn doesn't act super swell, so look for a PID."""
    if fabtools.files.is_file('/var/run/gunicorn/django.pid'):
        sudo('service gunicorn restart', pty=False)
    else:
        #fabtools.service.start('gunicorn')
        sudo('service gunicorn start', pty=False)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

from fabric.api import env
from fabric.api import parallel
from fabric.api import roles
from fabric.api import task

import fabtools.deb
import fabtools.require


DEFAULT_USER='farmboy'


def _set_env_defaults():
    env.setdefault('farmboy_user', DEFAULT_USER)
    env.setdefault('farmboy_files', './files')
    env.skip_bad_hosts = True
    env.timeout = 2
    env.roledefs = {'apt': [], 'ci': [], 'proxy': [], 'vcs': [], 'web': []}


_set_env_defaults()


@roles('proxy', 'web')
@task
@parallel
def install_user(user=DEFAULT_USER):
    """Ensure our default user has been created on all hosts."""
    env.farmboy_user = user
    fabtools.require.user(user, shell='/bin/bash')
    fabtools.deb.update_index(quiet=False)



# vim: tabstop=4 shiftwidth=4 softtabstop=4
import fabtools.require

from farmboy import util


from fabric.api import env
from fabric.api import roles
from fabric.api import task


WEB_SERVER = '  server web%(i)s %(host)s:%(port)s maxconn 32'


@roles('proxy')
@task
def deploy():
    """Add HAproxy to the <proxy> host."""
    fabtools.require.deb.packages([
        'haproxy'
    ])

    servers = []
    for i, server in enumerate(env.roledefs['web']):
        server_s = WEB_SERVER % {'i': i,
                                 'host': server.split('@')[-1],
                                 'port': '80'}
        servers.append(server_s)

    fabtools.require.files.template_file(
        template_source   = util.files('haproxy/haproxy.base'),
        context  = {'servers': '\n'.join(servers)},
        path     = '/etc/haproxy/haproxy.cfg',
        owner    = 'root',
        group    = 'root',
        mode     = '644',
        use_sudo = True)

    fabtools.require.files.file(
        source   = util.files('haproxy/default'),
        path     = '/etc/default/haproxy',
        owner    = 'root',
        group    = 'root',
        mode     = '644',
        use_sudo = True)

    fabtools.require.service.restarted('haproxy')



from fabric.api import env
from fabric.api import parallel
from fabric.api import roles
from fabric.api import task
import fabtools.require

from farmboy import util

@roles('apt', 'ci', 'proxy', 'vcs', 'web')
@task
@parallel
def hosts():
  hostsfile = util.files('farmboy/hosts')
  hosts = []
  # TODO(termie): does this break if we use callable roledefs?
  for role, role_hosts in env.roledefs.iteritems():
    if len(role_hosts) == 1:
      hosts.append('%s %s' % (util.host(role_hosts[0]), role))
      continue

    for i, role_host in enumerate(role_hosts):
      hosts.append('%s %s-%s' % (util.host(role_host), role, i))

  hosts_str = '\n'.join(hosts)
  print hosts_str

  fabtools.require.files.template_file(
      template_source = hostsfile,
      context  = {'hosts': hosts_str},
      path     = '/etc/hosts',
      owner    = 'root',
      group    = 'root',
      mode     = '644',
      use_sudo = True)





"""An HTTP proxy that speaks SOCKS5 on the backend that can be run in-process.

The purpose of this is so that we can point at a proxy set up by a command
like `ssh -D mygatewayserver` when we do python HTTP_PROXY calls.

It is a limited implementation.

"""

# A bunch of this is based on https://code.google.com/p/socksipy-branch,
# one of many seemingly abandoned projects for dealing with SOCKS

import BaseHTTPServer
import httplib
import os
import socket
import StringIO
import struct
import threading

from fabric.api import env
from fabric.api import task

env.farmboy_socks_proxy = os.environ.get('SOCKS_PROXY', 'localhost:1080')


class Socks5Socket(socket.socket):
  HELO = struct.pack('BBB', 0x05, 0x01, 0x00)
  SETD = struct.pack('BBB', 0x05, 0x01, 0x00)

  def __init__(self, rdns=True):
    self.rdns = rdns
    socket.socket.__init__(
        self, family=socket.AF_INET, type=socket.SOCK_STREAM, proto=0)

  def recvall(self, count):
    data = self.recv(count)
    while len(data) < count:
      d = self.recv(count - len(data))
      if not d:
        raise Exception('connection closed unexpectedly')
      data = data + d
    return data

  def negotiate(self):
    self.sendall(self.HELO)

    chosenauth = self.recvall(2)
    if chosenauth[0] != '\x05':
      self.close()
      raise Exception('invalid data received: ' + repr(chosenauth))

    if chosenauth[1] != '\x00':
      raise NotImplemented('only no-auth is implemented: ' + repr(chosenauth))

  def set_destination(self, addr, port):
    req = self.SETD

    # If the given destination address is an IP address, we'll
    # use the IPv4 address request even if remote resolving was specified.
    try:
      ipaddr = socket.inet_aton(addr)
      req = req + '\x01' + ipaddr
    except socket.error:
      if self.rdns:
        # Resolve remotely
        ipaddr = None
        req = req + '\x03' + chr(len(addr)).encode() + addr.encode()
      else:
        # Resolve locally
        ipaddr = socket.inet_aton(socket.gethostbyname(addr))
        req = req + '\x01' + ipaddr

    req = req + struct.pack('>H', int(port))
    self.sendall(req)

    # Alright, let's see if that worked out
    resp = self.recvall(4)
    if resp[0] != '\x05':
      # we got something weird back
      sock.close()
      raise Exception('invalid data')
    elif resp[1] != '\x00':
      # connection failed
      self.close()
      raise Exception('connection failed: ' + repr(resp))
    elif resp[3] == '\x01':
      boundaddr = self.recvall(4)
    elif resp[3] == '\x03':
      resp = resp + self.recv(1)
      boundaddr = self.recvall(ord(resp[4]))
    else:
      self.close()
      raise Exception('invalid data: ' + repr(resp))

    boundport = struct.unpack('>H', self.recvall(2))[0]

    # NOTE(termie): None of my tests produced values for boundaddr and
    #               boundport that we needed to care about


class SocketSaver(object):
  def __init__(self, sock):
    self._sock = sock
    self._raw = StringIO.StringIO()
    self._fp = None

  def makefile(self, *args, **kw):
    self._fp = self._sock.makefile(*args, **kw)
    return self

  def read(self, *args, **kw):
    rv = self._fp.read(*args, **kw)
    self._raw.write(rv)
    return rv

  def readline(self, *args, **kw):
    rv = self._fp.readline(*args, **kw)
    self._raw.write(rv)
    return rv

  def getvalue(self):
    return self._raw.getvalue()

  def __getattr__(self, key):
    return getattr(self._fp, key)


class HttpToSocks5Factory(object):
  def __init__(self, addr, port):
    self.addr = addr
    self.port = port

  def __call__(self, *args, **kw):
    return HttpToSocks5Handler(self.addr, self.port, *args, **kw)


class HttpToSocks5Handler(BaseHTTPServer.BaseHTTPRequestHandler):
  def __init__(self, addr, port, *args, **kw):
    self.addr = addr
    self.port = port
    BaseHTTPServer.BaseHTTPRequestHandler.__init__(self, *args, **kw)

  def _rebuild_request(self):
    raw = self.raw_requestline + str(self.headers) + '\r\n'
    content_length = self.headers.get('Content-Length', 0)
    content = self.rfile.read(int(content_length))
    raw = raw + content
    return raw

  def _get_destination(self):
    host_header = self.headers.get('Host')
    parts = host_header.split(':')
    host = parts.pop(0)
    if parts:
      port = parts.pop(0)
    else:
      port = 80

    return host, port

  def do_SOCKS5(self):
    sock = Socks5Socket()
    sock.connect((self.addr, int(self.port)))
    sock.negotiate()

    # Figure out where we want to send the request
    host, port = self._get_destination()
    sock.set_destination(host, port)

    # Send the request through the proxy
    req = self._rebuild_request()
    sock.sendall(req)

    # Get the response
    buf = SocketSaver(sock)
    http_resp = httplib.HTTPResponse(buf)
    http_resp.begin()
    http_resp.read()

    resp = buf.getvalue()
    self.wfile.write(resp)

  do_GET = do_SOCKS5
  do_POST = do_SOCKS5
  do_HEAD = do_SOCKS5
  do_PUT = do_SOCKS5


def _real_serve(server, event):
  while not event.is_set():
    server.handle_request()


def serve(http_addr, http_port, proxy_addr, proxy_port):
  event = threading.Event()
  server = BaseHTTPServer.HTTPServer(
      (http_addr, http_port), HttpToSocks5Factory(proxy_addr, proxy_port))

  thread = threading.Thread(target=_real_serve, args=(server, event))
  thread.daemon = True
  thread.start()
  return event


@task
def proxy(host=None, port=None):
  if host is None:
    host, port = env.farmboy_socks_proxy.split(':')

  os.environ['HTTP_PROXY'] = 'http://localhost:1081'
  evt = serve('localhost', 1081, host, port)




# vim: tabstop=4 shiftwidth=4 softtabstop=4
import fabtools.nginx
import fabtools.require

from farmboy import util


from fabric.api import parallel
from fabric.api import roles
from fabric.api import task


@roles('web')
@task
@parallel
def deploy():
    """Add nginx to the the <web> hosts."""
    fabtools.require.nginx.server()
    fabtools.require.file(
        source   = util.files('nginx/web'),
        path     = '/etc/nginx/sites-available/web',
        owner    = 'root',
        group    = 'root',
        mode     = '755',
        use_sudo = True)

    fabtools.nginx.disable('default')
    fabtools.nginx.enable('web')
    fabtools.require.service.restarted('nginx')



# vim: tabstop=4 shiftwidth=4 softtabstop=4

import fabtools.deb
import fabtools.require

from farmboy import util


from fabric.api import env
from fabric.api import parallel
from fabric.api import roles
from fabric.api import task



def _set_env_defaults():
    env.setdefault('farmboy_apt_proxy', 'http://192.168.33.13:3142')


_set_env_defaults()


@roles('apt')
@task
def deploy():
    """Deploy apt-cacher to the <apt> host."""
    fabtools.require.deb.packages([
        'apt-cacher'
    ])

    fabtools.require.files.file(
        source   = util.files('apt-cacher/default'),
        path     = '/etc/default/apt-cacher',
        owner    = 'root',
        group    = 'root',
        mode     = '644',
        use_sudo = True)

    fabtools.require.files.file(
        source   = util.files('apt-cacher/apt-cacher.conf'),
        path     = '/etc/apt-cacher/apt-cacher.conf',
        owner    = 'root',
        group    = 'root',
        mode     = '644',
        use_sudo = True)

    fabtools.require.service.restarted('apt-cacher')


@roles('ci', 'proxy', 'vcs', 'web')
@task
@parallel
def set_proxy(proxy=None):
    """Set the <apt> host as the proxy for apt on <all> hosts."""
    if proxy is None:
        proxy = env.farmboy_apt_proxy

    fabtools.require.files.template_file(
        template_source   = util.files('apt-cacher/01apt-cacher'),
        context  = {'proxy': proxy},
        path     = '/etc/apt/apt.conf.d/01apt-cacher',
        owner    = 'root',
        group    = 'root',
        mode     = '644',
        use_sudo = True)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

#import shutil

from farmboy import util

from fabric.api import env
from fabric.api import local
from fabric.api import task


DEFAULT_ROLEDEFS = {
    'proxy': ['vagrant@192.168.33.10'],
    #'vcs': ['vagrant@192.168.33.11'],
    #'ci': ['vagrant@192.168.33.12'],
    'apt': ['vagrant@192.168.33.13'],
    'web': ['vagrant@192.168.33.100',
            'vagrant@192.168.33.101'],
    'tomcat': ['vagrant@192.168.33.100',
               'vagrant@192.168.33.101']}

DEFAULT_APT_PROXY = 'http://192.168.33.13:3142'

KEYFILE_S = "os.path.expanduser('~/.vagrant.d/insecure_private_key')"


@task
def init():
    """Locally set up basic files for using Vagrant."""
    vagrantfile = util.files('vagrant/Vagrantfile')

    # TODO(termie): local cp or shutil?
    local('cp %s %s' % (vagrantfile, './Vagrantfile'))
    #shutil.copy(vagrantfile, './Vagrantfile')
    fabfile_context = {'roledefs': repr(DEFAULT_ROLEDEFS),
                       'keyfile': KEYFILE_S,
                       'preamble': '',
                       }
    util.template_file('farmboy/fabfile.py.template',
                       'fabfile.py',
                       fabfile_context)


@task
def build():
    """Launch and prepare Vagrant instances to be used by Farm Boy."""
    local('vagrant up')



# vim: tabstop=4 shiftwidth=4 softtabstop=4
import fabtools.nginx
import fabtools.require

from fabric.api import parallel
from fabric.api import roles
from fabric.api import task


@roles('web')
@task
@parallel
def deploy():
    """Add tomcat to the <web> hosts."""
    fabtools.require.deb.packages([
        'tomcat7'
    ])



import time

try:
  import boto.exception
  from boto import ec2
except ImportError:
  # We'll warn the user if they try to do anything
  pass

import yaml

from farmboy import util

from fabric.api import env
from fabric.api import execute
from fabric.api import task


env.farmboy_aws_region = 'us-west-2'
env.farmboy_aws_security_group = 'farmboy'
env.farmboy_aws_key_pair = 'farmboy'
env.farmboy_aws_image_id = 'ami-1e0c902e' #us-west-2 raring amd64
env.farmboy_aws_image_user = 'ubuntu'


DEFAULT_ROLEDEF_FILE = 'farmboy.aws.yaml'
DEFAULT_KEYFILE = 'farmboy.pem'
DEFAULT_ROLEDEFS = """util.load_roledefs('%s')""" % DEFAULT_ROLEDEF_FILE
DEFAULT_PREAMBLE = """

# The AWS utilities can launch your VMs for you using `farmboy aws.build`.
# This setting tells them how many and which roles to use, it will cache
# their addresses by default into the farmboy.aws.yaml file so that we
# can target them for the rest of our actions.
# POWER TIP: Use `farmboy aws.refresh` to update the farmboy.aws.yaml
#            file if you've manually changed your instances on EC2.
env.farmboy_aws_instances = ['proxy', 'apt', 'web', 'web']

# These are the various AWS-related configuration options and their default
# values. They should be pretty self explanatory for people who've used AWS.

# env.farmboy_aws_region = %(region)s
# env.farmboy_aws_security_group = %(security_group)s
# env.farmboy_aws_key_pair = %(key_pair)s
# env.farmboy_aws_image_id = %(image_id)s  # us-west-2 raring amd64
# env.farmboy_aws_image_user = %(image_user)s

# We're using `boto` for the built-in EC2 tools, so if you use those
# you should define the common AWS environment variables in your shell:
#
#   AWS_ACCESS_KEY_ID
#   AWS_SECRET_ACCESS_KEY

""" % {'region': repr(env.farmboy_aws_region),
       'security_group': repr(env.farmboy_aws_security_group),
       'key_pair': repr(env.farmboy_aws_key_pair),
       'image_id': repr(env.farmboy_aws_image_id),
       'image_user': repr(env.farmboy_aws_image_user)}


@task
@util.requires('boto', 'boto')
def init():
  """Locally set up basic files for using AWS."""
  fabfile_context = {'roledefs': DEFAULT_ROLEDEFS,
                     'keyfile': repr('%s.pem' % env.farmboy_aws_key_pair),
                     'preamble': DEFAULT_PREAMBLE}
  util.template_file('farmboy/fabfile.py.template',
                     'fabfile.py',
                     fabfile_context)


@task
@util.requires('boto', 'boto')
def build():
  """Launch and prepare AWS instances to be used by Farm Boy.

  This will ensure that a security group and key pair exist.
  It will also terminate any running instances tagged with `farmboy`.
  """
  conn = ec2.connect_to_region(env.farmboy_aws_region)
  security_group = env.farmboy_aws_security_group

  # check for security group
  try:
    security_groups = conn.get_all_security_groups([security_group])
  except boto.exception.EC2ResponseError:
    security_groups = []

  if not security_groups:
    util.puts('[aws] Creating security group: %s' % security_group)
    security_group = conn.create_security_group(
        security_group, 'default security group for farmboy')

    rules = [dict(ip_protocol='tcp',
                  from_port='22',
                  to_port='22',
                  cidr_ip='0.0.0.0/0'),
             dict(ip_protocol='tcp',
                  from_port='80',
                  to_port='80',
                  cidr_ip='0.0.0.0/0'),
             dict(ip_protocol='tcp',
                  from_port='8000',
                  to_port='9000',
                  cidr_ip='0.0.0.0/0'),
             dict(ip_protocol='udp',
                  from_port='60000',
                  to_port='61000',
                  cidr_ip='0.0.0.0/0'),
             dict(ip_protocol='tcp',
                  from_port='3142',
                  to_port='3142',
                  cidr_ip='0.0.0.0/0')]

    for rule in rules:
      util.puts('[aws]   adding rule: %(ip_protocol)s:%(from_port)s-%(to_port)s'
                ' (%(cidr_ip)s)' % rule)
      security_group.authorize(**rule)


  security_groups = [security_group]

  # check for keypair
  key_pair = conn.get_key_pair(env.farmboy_aws_key_pair)
  if not key_pair:
    util.puts('[aws] Creating key pair: %s' % env.farmboy_aws_key_pair)
    key_pair = conn.create_key_pair(env.farmboy_aws_key_pair)
    key_pair.save('./')

  execute(terminate)

  machines = env.farmboy_aws_instances

  # launch instances with in security group, with keypair
  util.puts('[aws] Starting %d instances' % len(machines))
  reservation = conn.run_instances(image_id=env.farmboy_aws_image_id,
                                   min_count=len(machines),
                                   max_count=len(machines),
                                   key_name=env.farmboy_aws_key_pair,
                                   security_groups=security_groups)

  # get instances for returned reservation
  new_instances = reservation.instances[:]

  # tag the instances with their roles
  for machine in machines:
    inst = new_instances.pop()
    util.puts('[aws]   assigning role: %s -> %s' % (inst.id, machine))
    inst.add_tag('farmboy', machine)

  execute(refresh, expected=len(machines))


@task
@util.requires('boto', 'boto')
def terminate():
  """Terminate running AWS instances tagged with `farmboy`."""
  conn = ec2.connect_to_region(env.farmboy_aws_region)
  # check for instances
  running_instances = conn.get_only_instances(
      filters={'tag-key': 'farmboy',
               'instance-state-name': 'pending',
               'instance-state-name': 'running'})

  # TODO(termie): we _probably_ want to just terminate them all and start
  #               new ones, is there a use case for keeping them?
  for instance in running_instances:
    util.puts('[aws] Terminating running instance: %s (%s)'
              % (instance.id, instance.tags['farmboy']))
    instance.terminate()


@task
@util.requires('boto', 'boto')
def refresh(expected=None):
  """Update local cache of IPs for AWS instances.

  This will write a `farmboy.aws.yaml` file of the running instances
  on AWS tagged with `farmboy` and their associated roles.
  """
  conn = ec2.connect_to_region(env.farmboy_aws_region)

  max_tries = 15
  for i in range(max_tries):
    util.puts('[aws] Fetching running and pending instances')
    running_instances = conn.get_only_instances(
        filters={'tag-key': 'farmboy',
                 'instance-state-name': 'pending',
                 'instance-state-name': 'running'})

    if not expected:
      if running_instances:
        break
      util.puts('[aws]   nothing found, retrying in 5 seconds (%d of %d)'
                % (i, max_tries))
    else:
      if running_instances and len(running_instances) == expected:
        break
      util.puts('[aws]   found %d, expecting %d, retrying in 5 seconds'
                ' (%d of %d)'
                % (len(running_instances), expected, i, max_tries))

    time.sleep(5)

  if not running_instances or (expected and len(running_instances) != expected):
    raise Exception('Did not find enough running instances.')


  o = {'roledefs': {}}
  for inst in running_instances:
    role = str(inst.tags['farmboy'])
    util.puts('[aws]   found: %s (%s)' % (inst.ip_address, role))
    role_l = o['roledefs'].get(role, [])
    role_l.append(str('%s@%s' % (env.farmboy_aws_image_user, inst.ip_address)))

    o['roledefs'][role] = role_l

  util.puts('[aws] Dumping roledefs to file: %s' % DEFAULT_ROLEDEF_FILE)
  yaml.dump(o,
            stream=open(DEFAULT_ROLEDEF_FILE, 'w'),
            default_flow_style=False,
            indent=2,
            width=72)




from setuptools import setup

config = dict(
    name='farmboy',
    version='0.1.0',
    url='https://github.com/termie/farmboy',
    description='Rapid development environments.',
    author='Andy Smith',
    author_email='github@anarkystic.com',
    install_requires=['fabric', 'fabtools', 'pyyaml'],
    extras_require={
        'openstack': ['novaclient'],
        'aws': ['boto'],
    },
    package_data={'farmboy': ['_files/*/*']},
    packages=['farmboy'],
    entry_points={
        'console_scripts': [
            'farmboy = farmboy.cli:main',
        ],
    },
)

setup(**config)



import os
import time

try:
  from novaclient.v1_1 import client
except ImportError:
  # We'll warn the user if they try to do anything
  pass

import yaml

from farmboy import util

from fabric.api import env
from fabric.api import execute
from fabric.api import local
from fabric.api import task

env.farmboy_os_username = os.environ.get('OS_USERNAME', '')
env.farmboy_os_password = os.environ.get('OS_PASSWORD', '')
env.farmboy_os_tenant_name = os.environ.get('OS_TENANT_NAME', '')
env.farmboy_os_auth_url = os.environ.get('OS_AUTH_URL', '')

env.farmboy_os_image_id = '89bd93bd-9523-422d-8e1d-9a301e599df1' # ubuntu 13.04
env.farmboy_os_image_user = 'ubuntu'
env.farmboy_os_flavor_id = '2' #m1.small
env.farmboy_os_reservation_id = 'farmboy'
env.farmboy_os_security_group = 'farmboy'
env.farmboy_os_keypair = 'farmboy'
env.farmboy_os_keyfile = 'farmboy.key'
env.farmboy_os_keyfile_public = 'farmboy.key.pub'

DEFAULT_ROLEDEF_FILE = 'farmboy.os.yaml'
DEFAULT_KEYFILE = 'farmboy.pem'
DEFAULT_ROLEDEFS = """util.load_roledefs('%s')""" % DEFAULT_ROLEDEF_FILE
DEFAULT_PREAMBLE = """

# The Openstack utilities can launch your VMs for you using
# `farmboy openstack.build`. This setting tells them how many and which roles
# to use, it will cache their addresses by default into the farmboy.os.yaml
# file so that we can target them for the rest of our actions.
# POWER TIP: Use `farmboy openstack.refresh` to update the farmboy.os.yaml
#            file if you've manually changed your instances.
env.farmboy_os_instances = ['proxy', 'apt', 'web', 'web']

# We generally want to use a stock Ubuntu 13.04 cloud image, you'll need
# to figure out the ID of one (or upload it using the dashboard).
# POWER TIP: Or you can use our handy picker!
env.farmboy_os_image_id = util.load('farmboy_os_image_id', '%(default_yaml)s')

# These are the various OpenStack-related configuration options and their
# default values. They should be pretty self explanatory for people who've
# used OpenStack.

# env.farmboy_os_username = os.environ.get('OS_USERNAME', '')
# env.farmboy_os_password = os.environ.get('OS_PASSWORD', '')
# env.farmboy_os_tenant_name = os.environ.get('OS_TENANT_NAME', '')
# env.farmboy_os_auth_url = os.environ.get('OS_AUTH_URL', '')

# env.farmboy_os_image_user = 'ubuntu'
# env.farmboy_os_flavor_id = '2' #m1.small, usually
# env.farmboy_os_reservation_id = 'farmboy'
# env.farmboy_os_security_group = 'farmboy'
# env.farmboy_os_keypair = 'farmboy'
# env.farmboy_os_keyfile = 'farmboy.key'
# env.farmboy_os_keyfile_public = 'farmboy.pub'


# We're using `python-novaclient` for the built-in OpenStack tools, so if you
# use those you should define the common OpenStack environment variables in
# your shell:
#
#   OS_USERNAME
#   OS_PASSWORD
#   OS_TENANT_NAME
#   OS_AUTH_URL

""" % {'default_yaml': 'farmboy.os.yaml'}


@task
def build_keypair():
  if not os.path.exists(env.farmboy_os_keyfile):
    local('ssh-keygen -q -N "" -t rsa -f %s' % env.farmboy_os_keyfile)


@task
@util.requires('novaclient', 'novaclient')
def init():
  """Locally set up basic files for using AWS."""
  fabfile_context = {'roledefs': DEFAULT_ROLEDEFS,
                     'keyfile': repr('%s.pem' % env.farmboy_aws_key_pair),
                     'preamble': DEFAULT_PREAMBLE}
  util.template_file('farmboy/fabfile.py.template',
                     'fabfile.py',
                     fabfile_context)


@task
@util.requires('novaclient', 'novaclient')
def build():
  """Launch and prepare OpenStack instances to be used by Farm Boy.

  This will ensure that a security group and key pair exist.
  It will also terminate any running instances tagged with `farmboy`.
  """
  conn = client.Client(env.farmboy_os_username,
                       env.farmboy_os_password,
                       env.farmboy_os_tenant_name,
                       env.farmboy_os_auth_url,
                       service_type='compute')

  security_group_name = env.farmboy_os_security_group

  # check for security group
  security_group = None
  security_groups = conn.security_groups.list()
  for group in security_groups:
    if group.name == security_group_name:
      security_group = group

  if not security_group:
    util.puts('[os] Creating security group: %s' % security_group)
    security_group = conn.security_groups.create(
        security_group_name, 'default security group for farmboy')

    rules = [dict(ip_protocol='tcp',
                  from_port='22',
                  to_port='22',
                  cidr='0.0.0.0/0'),
             dict(ip_protocol='tcp',
                  from_port='80',
                  to_port='80',
                  cidr='0.0.0.0/0'),
             dict(ip_protocol='tcp',
                  from_port='8000',
                  to_port='9000',
                  cidr='0.0.0.0/0'),
             dict(ip_protocol='udp',
                  from_port='60000',
                  to_port='61000',
                  cidr='0.0.0.0/0'),
             dict(ip_protocol='tcp',
                  from_port='3142',
                  to_port='3142',
                  cidr='0.0.0.0/0')]

    for rule in rules:
      util.puts('[os]   adding rule: %(ip_protocol)s:%(from_port)s-%(to_port)s'
                ' (%(cidr)s)' % rule)
      conn.security_group_rules.create(parent_group_id=security_group.id,
                                       **rule)

  security_groups = [security_group.name]

  # check for keypair
  keypair = None
  keypairs = conn.keypairs.list()
  for pair in keypairs:
    if pair.name == env.farmboy_os_keypair:
      keypair = pair

  if not keypair:
    execute(build_keypair)
    util.puts('[os] Creating keypair: %s' % env.farmboy_os_keypair)
    public_key = open(env.farmboy_os_keyfile_public).read()

    keypair = conn.keypairs.create(env.farmboy_os_keypair,
                                   public_key=public_key)

  execute(terminate)

  machines = env.farmboy_os_instances

  # launch instances with in security group, with keypair
  util.puts('[os] Starting %d instances' % len(machines))
  # tag the instances with their roles
  for machine in machines:
    inst = conn.servers.create(name='farmboy-%s' % machine,
                               image=env.farmboy_os_image_id,
                               flavor=env.farmboy_os_flavor_id,
                               security_groups=security_groups,
                               key_name=env.farmboy_os_keypair,
                               meta={'farmboy': machine})
    util.puts('[os]  started server: %s -> %s' % (inst.id, machine))

  execute(refresh, expected=len(machines))


@task
@util.requires('novaclient', 'novaclient')
def terminate():
  """Terminate running OpenStack instances tagged with `farmboy`."""
  conn = client.Client(env.farmboy_os_username,
                       env.farmboy_os_password,
                       env.farmboy_os_tenant_name,
                       env.farmboy_os_auth_url,
                       service_type='compute')

  # check for instances
  running_instances = conn.servers.list()

  # TODO(termie): we _probably_ want to just terminate them all and start
  #               new ones, is there a use case for keeping them?
  for instance in running_instances:
    if (instance.metadata.get('farmboy')
        and instance.status == 'ACTIVE'
        and getattr(instance, 'OS-EXT-STS:task_state') == None):
      util.puts('[os] Terminating running instance: %s (%s)'
                % (instance.id, instance.metadata['farmboy']))
      instance.stop()
      instance.delete()


@task
@util.requires('novaclient', 'novaclient')
def images(filter=None):
  """List the images available on the server."""
  conn = client.Client(env.farmboy_os_username,
                       env.farmboy_os_password,
                       env.farmboy_os_tenant_name,
                       env.farmboy_os_auth_url,
                       service_type='compute')
  images = conn.images.list()
  for i, image in enumerate(images):
    print '[%d] %s (%s)' % (i, image.name, image.id)

  print
  print 'Choose an image from above, or leave blank to skip.'
  print '(It will be written to %s)' % DEFAULT_ROLEDEF_FILE
  print
  number = raw_input('>>> ')

  if not number:
    return

  selected = images[int(number)]
  o = {'farmboy_os_image_id': str(selected.id)}
  util.update(o, DEFAULT_ROLEDEF_FILE)


@task
@util.requires('novaclient', 'novaclient')
def refresh(expected=None):
  """Update local cache of IPs for OpenStack instances.

  This will write a `farmboy.os.yaml` file of the running instances
  on OpenStack tagged with `farmboy` and their associated roles.
  """
  conn = client.Client(env.farmboy_os_username,
                       env.farmboy_os_password,
                       env.farmboy_os_tenant_name,
                       env.farmboy_os_auth_url,
                       service_type='compute')

  max_tries = 30
  for i in range(max_tries):
    util.puts('[os] Fetching running and pending instances')

    # check for instances
    found_instances = []
    running_instances = conn.servers.list()

    # TODO(termie): we _probably_ want to just terminate them all and start
    #               new ones, is there a use case for keeping them?
    for instance in running_instances:
      if (instance.metadata.get('farmboy')
          and instance.status == 'ACTIVE'
          and getattr(instance, 'OS-EXT-STS:task_state') == None):
        found_instances.append(instance)

    if not expected:
      if found_instances:
        break
      util.puts('[os]   nothing found, retrying in 5 seconds (%d of %d)'
                % (i, max_tries))
    else:
      if found_instances and len(found_instances) == expected:
        break
      util.puts('[os]   found %d, expecting %d, retrying in 5 seconds'
                ' (%d of %d)'
                % (len(found_instances), expected, i, max_tries))

    time.sleep(5)

  if not found_instances or (expected and len(found_instances) != expected):
    raise Exception('Did not find enough running instances.')


  o = {'roledefs': {}}
  for inst in found_instances:
    role = str(inst.metadata['farmboy'])
    ip_address = inst.addresses['private'][0]['addr']
    util.puts('[os]   found: %s (%s)' % (ip_address, role))
    role_l = o['roledefs'].get(role, [])
    role_l.append(str('%s@%s' % (env.farmboy_os_image_user, ip_address)))

    o['roledefs'][role] = role_l

  util.puts('[os] Dumping roledefs to file: %s' % DEFAULT_ROLEDEF_FILE)
  util.update(o, DEFAULT_ROLEDEF_FILE)
